#!/usr/bin/env python

import optparse
import re
import sys
import logging
import json

logging.basicConfig(level=logging.WARN, format='\033[1m%(asctime)s %(levelname)s : %(message)s\033[0m')

pattern = re.compile('<script id="results-json" type="application/json">(.*?)</script>')
def extractJsonFromHtml(html_src):
  m = pattern.search(html_src)
  if m is None:
    return None
  return m.group(1)

def replaceJsonInHtml(html_src, json_src):
  m = pattern.search(html_src)
  return html_src[:m.start()] + '<script id="results-json" type="application/json">' + json_src + '</script>' + html_src[m.end():]

def loadJsonData(input):
  data = None
  if input is None:
    logging.info('input is stdin')
    data = sys.stdin.read()
  else:
    try:
      f = open(input, 'r')
      data = f.read()
    except IOError as e:
      logging.error("I/O error({0}): {1}".format(e.errno, e.strerror))
      raise
  json_src = extractJsonFromHtml(data)
  if json_src is None:
    json_src = data
  try:
    json_data = json.loads(json_src)
  except:
    return None
  assert isinstance(json_data, list)
  return json_data

def mergeResultSet(a, b):
  output_tests = {}
  test_set = set(a['tests'].keys()) | set(b['tests'].keys())
  for test in test_set:
    metrics_set = set()
    if test in a['tests']:
      metrics_set |= set(a['tests'][test]['metrics'].keys())
    if test in b['tests']:
      metrics_set |= set(b['tests'][test]['metrics'].keys())
    output_tests[test] = {
      'metrics': {}
    }
    for metric in metrics_set:
      values = []
      units = None
      important = None
      if test in a['tests'] and metric in a['tests'][test]['metrics']:
        current_values = a['tests'][test]['metrics'][metric]['current']
        if current_values != None:
          values += current_values
        units = a['tests'][test]['metrics'][metric]['units']
        important = a['tests'][test]['metrics'][metric]['important']
      if test in b['tests'] and metric in b['tests'][test]['metrics']:
        current_values = b['tests'][test]['metrics'][metric]['current']
        if current_values != None:
          values += current_values
        if units is None:
          units = b['tests'][test]['metrics'][metric]['units']
          important = b['tests'][test]['metrics'][metric]['important']
        else:
          assert units == b['tests'][test]['metrics'][metric]['units']
          assert important == b['tests'][test]['metrics'][metric]['important']
      output_tests[test]['metrics'][metric] = {
        'current': values,
        'units': units,
        'important': important
      }
  return {
    'label': a['label'],
    'platform': a['platform'],
    'buildTime': a['buildTime'],
    'tests': output_tests
  }

def mergeResultSetsWithSameName(json_data):
  result_sets_by_name = {}
  for result_set in json_data:
    name = result_set['label']
    if name in result_sets_by_name:
      result_sets_by_name[name] = mergeResultSet(result_sets_by_name[name], result_set)
    else:
      result_sets_by_name[name] = result_set
  return result_sets_by_name.values()
